==> ordering-core-1.5.0-beta.1.tgz: package/lib/esm/auth.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from 'axios';
import { handleAPIError } from './apiTools';
import { Mutex } from './utils/mutex';
let _authData = null;
let tokenRetrieveTimeMs = -1;
const mutex = new Mutex(250);
export function authorizeWithUserPass(baseUrl, tenant, basicAuthPass, username, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios({
                method: 'post',
                url: `${baseUrl}/auth-oauth2/oauth/token`,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
                data: `username=${username}&password=${password}&grant_type=password&scope=read`,
            });
            _authData = response.data;
            tokenRetrieveTimeMs = new Date().getTime();
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error);
            return false;
        }
    });
}
export function authorizeWithRefreshToken(baseUrl, tenant, basicAuthPass, refreshToken) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            const req = {
                method: 'post',
                url: `${baseUrl}/auth-oauth2/oauth/token`,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
                data: `refresh_token=${refreshToken}&grant_type=refresh_token`,
            };
            //console.log(req);
            response = yield axios(req);
            _authData = response.data;
            tokenRetrieveTimeMs = new Date().getTime();
            //console.log('REFRESH TOKEN AUTH RESULT ***');
            //console.log(_authData);
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error)
            return false;
        }
    });
}
function isTokenExpired(authData) {
    // check if we can use current token
    if (!authData || !authData.access_token) {
        return true;
    }
    const secondsFromRetrievingExistingToken = new Date().getTime() - tokenRetrieveTimeMs;
    const expiryIfGreaterThan = parseInt(authData.expires_in) * 1000 * 0.95; //miliseconds
    return secondsFromRetrievingExistingToken >= expiryIfGreaterThan;
}
export const authDataProvider = (ctx, refreshTokenStorageHandler, forceRefresh = false) => __awaiter(void 0, void 0, void 0, function* () {
    // console.log( `authDataProvider invoked ----     currentToken=${  _authData ? _authData.access_token : '(null)'   }`, );
    if (!forceRefresh && _authData && _authData.access_token) {
        if (!isTokenExpired(_authData)) {
            return { token: _authData.access_token, UUID: _authData.UUID };
        }
    }
    //console.log('******** GET AUTH TOKEN (REMOTE CALL) ********');
    let loginSuccess = false;
    const refreshToken = yield refreshTokenStorageHandler.getRefreshToken(ctx.TENANT);
    if (refreshToken) {
        //console.log('*** AUTHORIZE WITH REFRESH TOKEN ');
        // use mutex to enforce max 1 api call at a time
        loginSuccess = yield mutex.getLock(() => {
            if (_authData && !isTokenExpired(_authData)) {
                return Promise.resolve(true);
            }
            return authorizeWithRefreshToken(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, refreshToken);
        });
    }
    if (!loginSuccess && ctx.anonymousAuth) {
        //console.log('**** ANONYMOUS AUTH ****');
        loginSuccess = yield authorizeWithUserPass(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, 'anonymous', '');
    }
    if (!loginSuccess || !_authData) {
        return { token: '', UUID: '' };
    }
    else {
        refreshTokenStorageHandler.setRefreshToken(ctx.TENANT, _authData.refresh_token);
        return { token: _authData.access_token, UUID: _authData.UUID };
    }
});
export function createAuthDataProvider(ctx, refreshTokenProvider, forceRefresh = false) {
    return () => authDataProvider(ctx, refreshTokenProvider, forceRefresh);
}
export function setAuthData(tenant, newAuthData, refreshTokenStorageHandler, newTokenRetrieveTimeMs = new Date().getTime()) {
    _authData = newAuthData;
    tokenRetrieveTimeMs = newTokenRetrieveTimeMs;
    refreshTokenStorageHandler.setRefreshToken(tenant, _authData.refresh_token);
}
export function clearAuthData(tenant, refreshTokenStorageHandler) {
    refreshTokenStorageHandler.clearRefreshToken(tenant);
    _authData = null;
}
export function getLoggedUserData(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios({
            method: 'get',
            url: `${baseURL}/auth-api/api/me`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function updateUserData(baseURL, token, userData) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios
            .post(`${baseURL}/auth-api/api/me`, userData, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
export function deleteUser(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const { login } = yield getLoggedUserData(baseURL, token);
        const response = yield axios
            .request({
            url: `${baseURL}/auth-api/api/me`,
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: { login },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}

==> ordering-core-1.5.2-beta.1.tgz: package/lib/esm/auth.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios, { isAxiosError } from 'axios';
import { handleAPIError } from './apiTools';
import { Mutex } from './utils/mutex';
import { jwtDecode } from 'jwt-decode';
let _authData = null;
const mutex = new Mutex(250);
export function authorizeWithUserPass(baseUrl, tenant, basicAuthPass, username, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { username, password, grant_type: 'password', scope: 'read' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error);
            return false;
        }
    });
}
const invalidRefreshTokens = new Map();
export function authorizeWithRefreshToken(baseUrl, tenant, basicAuthPass, refreshToken) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { refresh_token: refreshToken, grant_type: 'refresh_token' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            // console.log('authData', _authData);
            //console.log('REFRESH TOKEN AUTH RESULT ***');
            //console.log(_authData);
            return true;
        }
        catch (error) {
            if (isAxiosError(error) && /^4\d{2}$/.test(`${(_a = error.response) === null || _a === void 0 ? void 0 : _a.status}`)) {
                invalidRefreshTokens.set(refreshToken, Date.now());
            }
            //console.error('Authorization error');
            // console.error(error);
            return false;
        }
    });
}
export function isTokenExpired(token) {
    // check if we can use current token
    if (token) {
        try {
            return (
            // 10s before actual expiry
            jwtDecode(token).exp * 1000 < new Date().getTime() + 10000);
        }
        catch (error) {
            // Not a JWT token
            return true;
        }
    }
    return true;
}
function isJwtToken(token) {
    try {
        jwtDecode(token);
        return true;
    }
    catch (error) {
        return false;
    }
}
export const authDataProvider = (ctx_1, refreshTokenStorageHandler_1, ...args_1) => __awaiter(void 0, [ctx_1, refreshTokenStorageHandler_1, ...args_1], void 0, function* (ctx, refreshTokenStorageHandler, forceRefresh = false) {
    // console.log( `authDataProvider invoked ----     currentToken=${  _authData ? _authData.access_token : '(null)'   }`, );
    if (!forceRefresh && _authData && _authData.access_token) {
        if (!isTokenExpired(_authData.access_token)) {
            return { token: _authData.access_token, UUID: _authData.UUID };
        }
    }
    //console.log('******** GET AUTH TOKEN (REMOTE CALL) ********');
    let loginSuccess = false;
    const refreshToken = yield refreshTokenStorageHandler.getRefreshToken(ctx.TENANT);
    if (refreshToken &&
        (!isJwtToken(refreshToken) || !isTokenExpired(refreshToken))) {
        //console.log('*** AUTHORIZE WITH REFRESH TOKEN ');
        // use mutex to enforce max 1 api call at a time
        loginSuccess = yield mutex.getLock(() => {
            if (_authData && !isTokenExpired(_authData.access_token)) {
                return Promise.resolve(true);
            }
            // If same refresh token was invalid less then 30s ago skip api call
            const invalidTokenTimestamp = invalidRefreshTokens.get(refreshToken);
            if (invalidTokenTimestamp && Date.now() < invalidTokenTimestamp + 30000) {
                return Promise.resolve(false);
            }
            return authorizeWithRefreshToken(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, refreshToken);
        });
    }
    if (!loginSuccess && ctx.anonymousAuth) {
        // console.log('**** ANONYMOUS AUTH ****');
        loginSuccess = yield authorizeWithUserPass(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, 'anonymous', '');
    }
    if (!loginSuccess || !_authData) {
        refreshTokenStorageHandler.clearRefreshToken(ctx.TENANT);
        return { token: '', UUID: '' };
    }
    else {
        refreshTokenStorageHandler.setRefreshToken(ctx.TENANT, _authData.refresh_token);
        return { token: _authData.access_token, UUID: _authData.UUID };
    }
});
export function createAuthDataProvider(ctx, refreshTokenProvider, forceRefresh = false) {
    return () => authDataProvider(ctx, refreshTokenProvider, forceRefresh);
}
export function setAuthData(tenant, newAuthData, refreshTokenStorageHandler) {
    _authData = newAuthData;
    refreshTokenStorageHandler.setRefreshToken(tenant, _authData.refresh_token);
}
export function clearAuthData(tenant, refreshTokenStorageHandler) {
    refreshTokenStorageHandler.clearRefreshToken(tenant);
    _authData = null;
}
export function getLoggedUserData(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios({
            method: 'get',
            url: `${baseURL}/auth-api/api/me`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function updateUserData(baseURL, token, userData) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios
            .post(`${baseURL}/auth-api/api/me`, userData, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
export function deleteUser(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const { login } = yield getLoggedUserData(baseURL, token);
        const response = yield axios
            .request({
            url: `${baseURL}/auth-api/api/me`,
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: { login },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
